// --------------------------------------------------------------------
// Copyright (c) 2017 by FPGALOVER 
// --------------------------------------------------------------------
//                     web: http://www.fpgalover.com
//                     email: admin@fpgalover.com or hbecerra@ece.ubc.ca
// --------------------------------------------------------------------
//	Author: Holguer Andres Becerra Daza
// Module: Simple W5100 module interface
// --------------------------------------------------------------------

module w5100(   						// inputs:
                                    address,
                                    chipselect,
                                    clk,
                                    reset_n,
                                    write_n,
                                    writedata,

                                   // outputs:
                                    readdata,
												
											//w5100
											SS_n,
											SLCK,
											MOSI,
											MISO,
											wReset
												);
												
												
parameter FIFO_SIZE = 250;
output reg [ 31: 0] readdata;
input   [  2: 0] address;
input            chipselect;
input            clk;
input            reset_n;
input            write_n;
input   [ 31: 0] writedata;

output	SS_n;
output	SLCK;
output	MOSI;
input 	MISO;
output 	wReset;

wire             clk_en;



assign clk_en = 1;




reg SPI_RESET=0;
wire SPI_START;
wire SPI_FINISH;
reg SPI_FINISH_reg=0;
reg SPI_WR=0;
wire SPI_READY;
wire [15:0]SPI_ADDRESS;
wire [7:0]NIOS_TO_WIZNET;
wire [7:0]SPI_TO_NIOS;
reg [23:0]DATA_IN_SPI=0;


assign wReset=SPI_RESET;


wire wr_data=chipselect && ~write_n && (address[2:0] == 3'd0);


 always @(posedge clk or negedge reset_n)
    begin
      if (reset_n == 0)
          {SPI_WR,DATA_IN_SPI[23:0]} <= 0;
      else if (wr_data)
          {SPI_WR,DATA_IN_SPI[23:0]} <= writedata[24 : 0];
    end
	 
	 
 always @(posedge clk or negedge reset_n)
    begin
      if (reset_n == 0)
          SPI_RESET <= 0;
      else if (chipselect && ~write_n && (address[2:0] == 3'd1))
          SPI_RESET <= writedata[0];
    end
	  
  always @(*)
    begin
      if (reset_n == 0)
          readdata <= 0;
      else if (chipselect && clk_en && (address[2:0] == 3'd2))
          readdata <= {24'd0 | SPI_TO_NIOS[7:0]};
		else if (chipselect && clk_en && (address[2:0] == 3'd3))
		    readdata <= {31'd0 | SPI_FINISH};
		else 
			readdata <= 0;
		
    end	 
	 

//=======================================================
//  Structural coding
//=======================================================


async_trap_and_reset async_trap_and_reset_inst
(
	.async_sig(wr_data) ,	// input  async_sig_sig
	.outclk(clk) ,	// input  outclk_sig
	.out_sync_sig(SPI_START) ,	// output  out_sync_sig_sig
	.auto_reset(1'b1) ,	// input  auto_reset_sig
	.reset(1'b1) 	// input  reset_sig
);



SPI_MASTER SPI_MASTER_inst
(
	.WRITE_READ(SPI_WR) ,	// input  WRITE_READ_sig
	.DATA_IN(DATA_IN_SPI[23:0]) ,	// input [23:0] DATA_IN_sig
	.DATA_OUT(SPI_TO_NIOS[7:0]) ,	// output [7:0] DATA_OUT_sig
	.CLK_SPI(clk) ,	// input  CLK_SPI_sig
	
	.SS_n(SS_n) ,	// output  SS_n_sig
	.SLCK(SLCK) ,	// output  SLCK_sig
	.MOSI(MOSI) ,	// output  MOSI_sig
	.MISO(MISO), 	// input  MISO_sig
	
	.RDY(SPI_READY) ,	// output  RDY_sig
	.START(SPI_START) ,	// input  START_sig
	.SPI_IDLE(SPI_FINISH) 	// output  SPI_IDLE_sig
	
);		




endmodule
